FUNCTION AFields    && Fill an Array With Fields of DBF
PARAMETERS type_call, m_buffer
********************************************************************************
* PROGRAM: AFIELDS.PRG
*    DATE: 8/19/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S AFIELDS() FUNCTION
*
*  PARAM1: TYPE OF CALL
*  PARAM2: STRING BUFFER FROM PREVIOUS TYPE 1 CALL
* RETURNS: RETURN VALUES FOR EACH TYPE OF CALL ARE LISTED BELOW
*          1 - BUFFER CONTAINING FIRST OR NEXT LINE OF DBF STRUCTURE LISTING
*          2 - FIELD NAME FROM STRUCTURE LISTING BUFFER
*          3 - FIELD TYPE FROM STRUCTURE LISTING BUFFER
*          4 - FIELD WIDTH (NUMERIC) FROM STRUCTURE LISTING BUFFER
*          5 - FIELD DECIMALS (NUMERIC) FROM STRUCTURE LISTING BUFFER
*
********************************************************************************
  PRIVATE m_temp, m_count

  IF TYPE('m_handle') = 'U'
    PUBLIC m_handle
    m_handle = 0
  ENDIF

  DO CASE

    CASE type_call = 1
      * RETURN BUFFER CONTAINING FIRST OR NEXT LINE OF DBF STRUCTURE LISTING
      IF LEN(m_buffer) = 0
        IF LEN(DBF()) = 0
        RETURN ""
      ENDIF
      m_temp = set('CONSOLE')
      SET CONSOLE OFF
      LIST STRUCTURE TO FILE stepwrk.wrk
      IF m_temp = 'ON'
        SET CONSOLE ON
      ENDIF
      m_handle = fopen("STEPWRK.WRK",0)
      m_buffer = " "
      m_temp = 0
      m_count = 0
      DO WHILE m_count < 5
        m_temp = AT(CHR(13),m_buffer)
        IF m_temp = 0
          m_buffer = SPACE(101)
          m_buffer = fread(m_handle,100)
        ELSE
          m_count = m_count + 1
          m_buffer = SUBSTR(m_buffer,m_temp+1)
        ENDIF
      ENDDO
      m_buffer = SUBSTR(m_buffer,2) + fread(m_handle,54-LEN(m_buffer))
    ELSE
      m_buffer = SPACE(54)
      m_buffer = fread(m_handle,53)
    ENDIF
    IF LEN(m_buffer) = 53
      RETURN m_buffer
    ELSE
      m_count = fclose(m_handle)
      RETURN ""
    ENDIF

  CASE type_call = 2
    * RETURN FIELD NAME FROM STRUCTURE LISTING BUFFER
    RETURN TRIM(SUBSTR(m_buffer,8,10))

  CASE type_call = 3
    * RETURN FIELD TYPE FROM STRUCTURE LISTING BUFFER
    RETURN SUBSTR(m_buffer,20,1)

  CASE type_call = 4
    * RETURN FIELD WIDTH FROM STRUCTURE LISTING BUFFER
    RETURN VAL(SUBSTR(m_buffer,33,3))

  CASE type_call = 5
    * RETURN FIELD DECIMALS FROM STRUCTURE LISTING BUFFER
    RETURN VAL(SUBSTR(m_buffer,41,2))

ENDCASE

RETURN ""
*-- EOF: AFields(type_call, m_buffer)


FUNCTION AllTrim    && Trim Blanks Before and After a String
PARAMETER lc_string
*-----------------------------------------------------------------------------
* Synopsis:
* 
* ALLTRIM( <expC> )
* 
* The ALLTRIM() function returns the given character expression <expC> with
* all leading and trailing blanks removed.  If the <expC> is a numeric type,
* ALLTRIM() will return a trimed string back.  If a memo field is used, a null
* string is returned.  If a logical value is used, the string ".T." or ".F."
* will be returned.
* 
* See also:  LTRIM(), RTRIM(), TRIM()
* 
*-----------------------------------------------------------------------------
PRIVATE lc_return, lc_type

lc_type = TYPE("lc_string")
DO CASE
  CASE lc_type = "C"
    lc_return = LTRIM( RTRIM( lc_string ) )
  CASE lc_type $ "NF"
    lc_return = LTRIM( RTRIM( STR(lc_string) ) )
  CASE lc_type = "L"
    IF lc_string
      lc_return = "T"
    ELSE
      lc_return = "F"
    ENDIF
  OTHERWISE                          
    lc_return = ""
ENDCASE
RETURN ( lc_return )
*-- EOP: AllTrim( lc_string )


FUNCTION ASort      && Sort an Array of Values
PARAMETERS calltype, value
********************************************************************************
* PROGRAM: ASORT.PRG
*    DATE: 8/22/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S ASORT() FUNCTION
*
*  PARAM1: TYPE OF CALL (1-POST TO SORT; 2-RETRIEVE FROM SORT)
*  PARAM2: VALUE TO POST TO SORT FOR CALL TYPE 1
*          SEQUENTIAL NUMBER ON TYPE 2 CALLS (STARTING WITH 1)
* RETURNS: RETURNS NEXT SORT VALUE IN SEQUENCE FOR CALL TYPE 2
*
********************************************************************************
  PRIVATE mtemp, mtemp2

  IF TYPE('m_sortarr[1]') = 'U'
    IF calltype = 1
      PUBLIC array m_sortarr[50]
      PUBLIC m_sortcnt
      m_sortcnt = 1
      m_sortarr[1] = value
    RETURN m_sortcnt
  ELSE
    RETURN .F.
  ENDIF
ENDIF

IF calltype = 1
  mtemp2 = 0
  mtemp = 1
  DO WHILE mtemp <= m_sortcnt .AND. mtemp2 = 0
    IF value <= m_sortarr[mtemp]
      * FOUND WHERE IT GOES - MOVE REST DOWN AND INSERT IT
      mtemp2 = m_sortcnt
      DO WHILE mtemp2 >= mtemp
        m_sortarr[mtemp2+1] = m_sortarr[mtemp2]
        mtemp2 = mtemp2 - 1
      ENDDO
      mtemp2 = -1
      m_sortarr[mtemp] = value
    ENDIF
    mtemp = mtemp + 1
  ENDDO
  m_sortcnt = m_sortcnt + 1
  IF mtemp2 = 0
    * ADD IT TO THE END
    m_sortarr[m_sortcnt] = value
  ENDIF
  RETURN m_sortcnt
ENDIF

IF calltype = 2
  IF value > m_sortcnt
    RETURN ""
  ENDIF
  mtemp = m_sortarr[value]
  IF value >= m_sortcnt
    RELEASE m_sortcnt, m_sortarr
  ENDIF
  RETURN mtemp
ENDIF

RETURN .F.
*-- EOF: ASort( calltype, value )


FUNCTION Center     && Provide column needed to center a string in a width
PARAMETER lc_string, ln_width
*-----------------------------------------------------------------------------
* Synopsis:
* 
* CENTER( <expC>, <expN> )
* 
* The CENTER() function will return the starting column coordinate to center
* the <expC> string within a width of screen <expN>.  The width of the screen
* would normally be 80 colunms, but could just as well be the width of a 
* window.  If there is an error condition, the returned result will equal 0.
* If a numeric value is passed for the <expC> value, it will be converted
* to a string.
* 
* Examples:
* 
*   @ 15,center(string,80) say string    
* 
* Will center the <string> withing 80 columns
* 
*-----------------------------------------------------------------------------
PRIVATE lc_result, lc_type

IF .NOT. TYPE("ln_width") $ "FN"       && Force value to 0 for bad type
  lc_result = 0
ELSE

  lc_type = TYPE("lc_string")
  DO CASE
    CASE lc_type = "C"
      lc_width = (ln_width/2)-(LEN(lc_string)/2)
    CASE lc_type $ "NF"
      lc_width = (ln_width/2)-(LEN(ALLTRIM(STR(lc_string)))/2)
    CASE lc_type = "L"
      lc_width = (ln_width/2)-(1.5)    && .T. or .F. have fixed len of 3
    OTHERWISE                          
      lc_width = 0
  ENDCASE
ENDIF

IF lc_width < 0                        && Force negative values to 0
  lc_width = 0
ENDIF

RETURN ( lc_width )
*-- EOF: Center( lc_string, ln_width )


FUNCTION Chk4Num    && Check to see if a field is a numeric/float
PARAMETER pn_fld
*-------------------------------------------------------------------
* Make sure that the parameter is a numeric value
*-------------------------------------------------------------------
  PRIVATE ll_result

  ll_result = .F.             && Assume the worst, it's not numeric
  IF TYPE( PN_FLD ) $ "NF"
    ll_result = .T.
  ENDIF

RETURN( ll_result )
*-- EOF: Chk4Num( pn_fld )


FUNCTION Color      && Retrieve or Set Color Attributes
PARAMETERS set_color
*-----------------------------------------------------------------------------
* Synopsis:
* 
* COLOR( <expC> )
*  <expC> = NORMAL, HIGHLIGHT, MESSAGES, TITLES, BOX, INFORMATION, FIELDS, COLOR
*        or a variable with all colors store in it
*  Ver: dBASE 1.1
*
* The COLOR() function either returns or sets colors returned with the
* SET("attribute") setting
* If <expC> is a color string then null is returned otherwise the color
* setting is returned for one of dBASE's color options
*
* Example:
*
*   lc_prevcol = SET("ATTRIBUTE")    && Save current colors
*   SET COLOR TO w+/b,gr+/n          && Change standard colors
*   SET COLOR OF INFORMATION TO w+/r && Change INFORMATION colors
*   ...
*   lc_tmp = COLOR("lc_prevcol")     && Restore colors back to before
*
*   lc_high = COLOR("HIGHLIGHT")     && Capture color of the HIGHLIGHT
* 
* See Also: SET("attribute")
* 
* Author: Kirk Nason
*-----------------------------------------------------------------------------
PRIVATE color_num, color_str, cnt, old_talk
DO Talkoff

IF "" = set_color .OR. "HELP" $ UPPER(set_color) 
  ? "Try:"
  ? ""
  DO Talkon
  RETURN "COLOR; NORMAL, HIGHLIGHT, BORDER;  MESSAGES, TITLES, BOX, INFORMATION, FIELDS"
ENDIF

set_color = UPPER(set_color)
IF set_color $ "COLOR"
  *- Return standard, enhanced, border colors only
  RETURN LEFT( SET("attribute"), AT(" &", SET("attribute") ) )
ENDIF

*- Declare array to parse color options from SET("attr")
PRIVATE color_
DECLARE color_[8]
*- Determine if user is restoring colors vs. saving colors
IF " &" $ set_color
  color_str = ","+set_color+","                  && Restore color attributes
ELSE
  color_str = ","+SET("ATTRIBUTE")+","           && Save color attributes
ENDIF

*-- Stuff array with individual color setting

color_str = STUFF(color_str, AT(" &", color_str), 4, ",")
cnt = 1
DO WHILE cnt <= 8                                && Pick out individual colors
  color_str = SUBSTR(color_str, AT(",", color_str ) + 1 )
  color_[cnt] = SUBSTR(color_str, 1, AT(",", color_str ) - 1)
  cnt = cnt + 1
ENDDO

IF " &" $ set_color                              && Have the string from SET("attribute")
  *-- Set color back
  SET COLOR TO ,,&color_[3].
  SET COLOR OF NORMAL TO &color_[1].
  SET COLOR OF HIGHLIGHT TO &color_[2].
  SET COLOR OF MESSAGES TO &color_[4].
  SET COLOR OF TITLES TO &color_[5].
  SET COLOR OF BOX TO &color_[6].
  SET COLOR OF INFORMATION TO &color_[7].
  SET COLOR OF FIELDS TO &color_[8].
ELSE
  *-- Return color string requested
  DO CASE
  CASE set_color $ "NORMAL"
    color_num =  1
  CASE set_color $ "HIGHLIGHT"
    color_num =  2
  CASE set_color $ "BORDER"
    color_num =  3
  CASE set_color $ "MESSAGES"
    color_num =  4
  CASE set_color $ "TITLES"
    color_num =  5
  CASE set_color $ "BOX"
    color_num =  6
  CASE set_color $ "INFORMATION"
    color_num =  7
  CASE set_color $ "FIELDS"
    color_num =  8
  OTHERWISE
    set_color = " &"
  ENDCASE
ENDIF
DO Talkon
RETURN IIF(" &" $ set_color, "", color_[color_num])
*-- EOF: Color( set_color )


FUNCTION CurDir     && Return the current DOS directory
PARAMETER pc_drive
*-----------------------------------------------------------
* Returns the current DOS directory
*
* Synopsis:
*
* CURDIR( <expC> )
*
* Description:
*
* CURDIR() returns the current DOS directory in effect 
* on the drive identified by <expC>.  The value returned is
* a character string.  The current default drive is
* assumed (dBASE IV's SET DEFAULT drive) if <expC> is null.  
* CURDIR will returna null string if no <expC> drive exists.
*
* CURDIR() ignores the dBASE IV's SET PATH values.
*
* Examples:
*
*     ? CURDIR('')
*     
*     RUN CD \DBASE\SAMPLES
*     ? CURDIR('C')
*     
*     SET DEFAULT TO C:\
*     ? CURDIR('C')		&& root
*     
*     
*     IF .NOT. CURDIR('C') = 'DBASE\SAMPLES\'
*        SET DEFAULT TO C:
*        RUN CD \DBASE\SAMPLES
*     ENDIF
*
* In the second program example, CURDIR() looks to see if
* the current directory for the C drive is \DBASE\SAMPLES. 
* If not, it sets the default to that directory.
*
* Warning: CURDIR.BIN must be loaded before the call.  Use
*          the following procedure to load and release the
*          BIN file.
*
*          DO Load_Bin WITH "curdir"
*          ...  Code to execute CurDir()
*          DO Rele_Bin WITH "curdir"
*
*-----------------------------------------------------------
PRIVATE lc_return

  lc_return = ""

  IF glcurdir     && BIN file correctly LOADed with LOAD_BIN

    IF ISBLANK( pc_drive )
      IF glgetdrive
        pc_drive = CALL( "GETDRIVE", " " )
      ENDIF
    ENDIF


    lc_return = CALL( "CURDIR", ;
                LEFT( UPPER(pc_drive) , 1 ) + SPACE( 65 ) )

  ENDIF

RETURN( lc_return )
*-- EOF: CurDir( pc_drive )


PROCEDURE Dos
*--------------------------------------------------------------------
* NAME
*   DOS - Present a DOS prompt in a window.
*
* DESCRIPTION
*   DOS allows easy DOS access from the dBASE design surfaces.
*
*   When called, DOS will prompt the user for a DOS
*   command to run.  This can be either a resident DOS
*   command such as "dir", a program name, or a .BAT
*   file.  DOS will then run the DOS command
*   entered, and automatically return to dBASE when
*   finished.  DOT till restore the original screen.
*
* DEPENDENCIES
*    _StrInBox() function in OACC.PRG
*    _TalkMode() function in OACC.PRG
*
* LIMITATIONS
*   Also, be very careful when deleting any files from
*   within dBASE IV.  Some may be files that dBASE IV
*   needs in order to run.
*
* VERSION
*   dBASE IV 1.5
*
*--------------------------------------------------------------------
  PRIVATE ALL LIKE l?_*

  DO Talkoff
  ll_status = SET( "STATUS" ) = "ON"    && Initial STATUS setting

  SAVE SCREEN TO dos

  lc_command = "command.com"
  ln_command = StrInBox( 5, 6, ;
                          "Enter the DOS program to run",;
                          lc_command, ;
                          128, ;
                          30 ;
                       )

  SET STATUS OFF

  IF .NOT. ISBLANK( lc_command ) .AND. ln_command = 0

    ln_result = RUN( lc_command, .T. )

    *-- Have the user press a key to continue
    ll_console = SET( "CONSOLE" ) = "ON"
    SET CONSOLE ON
    SET TALK ON
    WAIT "Press any key to continue . . ."
    SET TALK OFF
    IF .NOT. ll_console
      SET CONSOLE OFF
    ENDIF

  ENDIF

  IF ll_status
    SET STATUS ON
  ENDIF
  RESTORE SCREEN FROM dos
  RELEASE SCREEN dos

  DO Talkon

RETURN 
*-- EOP: Dos


FUNCTION Dot        && Call Dot Prompt in a Window
*-----------------------------------------------------------------------------
*-- Notes......: Call Dot prompt in a window
*-----------------------------------------------------------------------------
   DO Talkoff
   SAVE SCREEN TO w_mc
   DO shadowg WITH 10,0,20,77
   DEFINE WINDOW w_mc from 10,0 TO 20,79 DOUBLE
   ACTIVATE WINDOW w_mc
   @ 8,0 SAY "Type: RESUME to Return"
   SUSPEND
   RELEASE WINDOW w_mc
   RESTORE SCREEN FROM w_mc
   RELEASE SCREEN w_mc
   DO Key_Vars
   KEYBOARD CHR(kn_Enter) + REPLICATE( CHR(kn_BakSpce), 5 )
   DO Rele_Vars
   DO Talkon
RETURN .T.
*-- EOF: Dot()


FUNCTION Ed         && Call Modify Command in a Window 
PARAMETER p_file
*-----------------------------------------------------------------------------
*-- Notes......: Call Modify Command in a window 
*-----------------------------------------------------------------------------
  DO Talkoff
  SAVE SCREEN TO w_mc
  DO shadowg WITH 10,5,20,74
  DEFINE WINDOW w_mc from 10,5 TO 20,74 DOUBLE
  MODIFY COMMAND (p_file) WINDOW w_mc
  RELEASE WINDOW w_mc
  RESTORE SCREEN FROM w_mc
  RELEASE SCREEN w_mc
  DO Key_Vars
  KEYBOARD CHR(kn_Enter) + REPLICATE( CHR(kn_BakSpce), 2 )
  KEYBOARD [("")]
  KEYBOARD CHR( kn_Home ) + REPLICATE( CHR(kn_Del), 4 )
  KEYBOARD CHR( kn_End ) + REPLICATE( CHR(kn_LeftArw), 2 )
  DO Rele_Vars
  DO Talkon
RETURN .T.   
*-- EOF: Ed( p_file )


FUNCTION GetDrive   && Return the current DOS drive
*-----------------------------------------------------------
* Returns the current DOS drive
*
* Synopsis:
*
* GETDRIVE( )
*
* Description:
*
* GETDRIVE() returns the current DOS drive.
*
* Examples:
*
*     ? GETDRIVE()
*     
* Warning: GETDRIVE.BIN must be loaded before the call.  Use
*          the following procedure to load and release the
*          BIN file.
*
*          DO Load_Bin WITH "getdrive"
*          ...  Code to execute GetDrive()
*          DO Rele_Bin WITH "getdrive"
*
*-----------------------------------------------------------
PRIVATE lc_return

  lc_return = ""

  IF glgetdrive   && BIN file correctly LOADed with LOAD_BIN

    lc_return = CALL( "GETDRIVE", " " )

  ENDIF

RETURN( lc_return )
*-- EOF: GetDrive()


FUNCTION Header     && Return Length of DBF Header
********************************************************************************
* PROGRAM: HEADER.PRG
*    DATE: 8/11/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S HEADER() FUNCTION
*
* RETURNS: RETURNS LENGTH OF THE HEADER AREA OF CURRENT DBF
*   (SEE RECSIZE() IN THE dBASE IV LANGUAGE REFERENCE)
*
********************************************************************************
  PRIVATE mfcount

  IF LEN(DBF()) = 0
  RETURN 0
ENDIF

lc_talk = TalkMode( lc_talk )     && Restore previous TALK setting

mfcount = 1
DO WHILE LEN(FIELD(mfcount)) > 0
  mfcount = mfcount + 1
ENDDO

lc_talk = TalkMode(.F.)
RETURN 32 * (mfcount - 1) + 35
*-- EOF: Header()


FUNCTION HtoI       && Convert Hex Byte String to Integer Value
PARAMETERS param1
PRIVATE position, result, value, power16

param1 = UPPER(param1)
position = LEN(param1)
result = 0
power16 = 1

DO WHILE position > 0
   value = SUBSTR(param1, position, 1)
   IF value >= '0' .AND. value <= '9'
      value = VAL(value)
   ELSE
      value = ASC(value) - ASC('A') + 10
   ENDIF
   result = result + value * power16
   position = position - 1
   power16 = power16 * 16
ENDDO

RETURN result
*-- EOF: HtoI( param1 )


FUNCTION IndexOrd   && Determine Order Number of Controlling Index
********************************************************************************
* PROGRAM: INDEXORD.PRG
*    DATE: 8/11/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S INDEXORD()
*
* RETURNS: NUMERIC POSITION OF CONTROLLING INDEX IN LIST OF OPEN INDEX FILES
*
********************************************************************************
  PRIVATE morder, mindex

  * GET NAME OF CONTROLLING INDEX OR TAG
  morder = order()
  IF LEN(morder) = 0
    RETURN 0
  ENDIF


  mindex = 1
*-- TAG RETURNS NAME OF INDEX OR TAG IN POSITION mindex
  DO WHILE tag(mindex) <> morder
    mindex = mindex + 1
  ENDDO

RETURN mindex
*-- EOF: IndexOrd()


FUNCTION IsInView   && Check to see if a field is in current DBF
PARAMETER p_field
*------------------------------------------------------------------------
* Synopsis:
*
* IsInView( <p_field> )
* 
* The ISINVIEW() function will determine if the <p_field> is in the
* current view.
*
* The ISINVIEW() function currently checks only the current open DBF
* file.
*
*------------------------------------------------------------------------
  PRIVATE ln_fldcntr, ll_result

  ll_result = .F.                   && Assume the worst, field not in DBF
  ln_fldcnt = 1                     && Counter for looping fields
  DO WHILE .t.                                                   

    lc_field = FIELD( ln_fldcnt )   && Get the current field name
    IF ISBLANK( lc_field )            && Check for end of field list
      EXIT                          && and bail out w/o setting ll_result
    ELSE
      IF RTRIM(p_field) = lc_field  && Check for match
        ll_result = .T.             && Found it, set the return flag TRUE
        EXIT                        && and bail out of DO WHILE loop
      ELSE
        ln_fldcnt = ln_fldcnt + 1   && Increment field counter
      ENDIF
    ENDIF

  ENDDO

RETURN( ll_result )
*-- EOF: IsInVIew( p_field )


FUNCTION IsTime     && Determine if a String is a Time String
PARAMETER str_time
*-------------------------------------------------------------------------
* Synopsis:
* 
* ISTIME( <expCTIME> )
* 
* The ISTIME() function determines if the <ExpCTIME> expression is a 
* properly formatted dBASE IV time string and return .T. if it is.  The
* character used to delimit hours from minutes from seconds is ignored.
* Whatever the delimiter is, it must be in colunm positions 3 and 6.
* Leading zeros are required for the time string to pass the midnight
* condition.
* 
*-----------------------------------------------------------------------------
PRIVATE lc_result, lc_hour, lc_minute, lc_second

lc_result = .F.                        && Assume bogus time string
lc_hour   = 0 
lc_minute = 0
lc_second = 0

IF TYPE("str_time") = "C" .AND. LEN(str_time) = 8

  lc_hour = VAL( SUBSTR( str_time, 1, 2 ) )
  IF lc_hour >= 0 .AND. lc_hour < 24

    lc_minute = VAL( SUBSTR( str_time, 4, 2 ) )
    IF lc_minute >= 0 .AND. lc_minute < 60

      lc_second = VAL( SUBSTR( str_time, 7, 2 ) )
      IF lc_second >= 0 .AND. lc_minute < 60

        lc_result = .T.

      ENDIF

    ENDIF

  ENDIF

ENDIF

IF lc_hour = 0 .AND. lc_minute = 0 .AND. lc_second = 0 .AND. ;
   SUBSTR( str_time, 1, 2 ) <> "00"
  lc_result = .F.
ENDIF

RETURN ( lc_result )
*-- EOP: istime( str_time )


FUNCTION IsWhite    && Determine if String Fully Contains White Space
PARAMETER chkstr
*-------------------------------------------------------------------------
* Synopsis:
* 
* ISWHITE( <expC1> )
* 
* The ISWHITE() function determines if all of the characters in <expC1> 
* are white space characters, and then returns .T. if they are.  
* White space is defined as a space, tab, carriage return, formfeed, or 
* linefeed characters.  If all of the characters are not white space, or
* an invalid function argement is used, a .F. is returned.
*-------------------------------------------------------------------------
PRIVATE l_chkstr                       && Length of parameter
PRIVATE cur_pos                        && Current position in chkstr
PRIVATE white_sp                       && String of white space characters
PRIVATE rtn_val                        && Return value

*-- Check arguments
IF .NOT. TYPE("chkstr") $ "MC"
  RETURN ( .F. )
ENDIF

white_sp = CHR( 13)                    && Normal carriage return
white_sp = white_sp + CHR(141)         && Memo field carriage return
white_sp = white_sp + CHR( 32)         && Space character
white_sp = white_sp + CHR( 10)         && Line feed
white_sp = white_sp + CHR( 09)         && Horizontal Tab
white_sp = white_sp + CHR( 12)         && Form Feed

*-- Get the length of chkstr to avoid computing it all of the time
l_chkstr = LEN(chkstr)

rtn_val = .T.                          && Assume innocent until proven guilty
cur_pos = 1
DO WHILE cur_pos <= l_chkstr
  IF .NOT. SUBSTR( chkstr, cur_pos, 1 ) $ white_sp
    rtn_val = .F.                      && Character was not white space
    EXIT                               && Bail out of DO WHILE
  ENDIF
  cur_pos = cur_pos + 1
ENDDO

RETURN ( rtn_val )
*-- EOP: IsWhite( chkstr )


FUNCTION ItoH       && Convert Integer Value to Hex Byte
PARAMETERS param1, param2
********************************************************************************
* PROGRAM: ITOH.PRG
*    DATE: 8/31/88
*      BY: DON KELLER
* PURPOSE: USER DEFINED FUNCTION TO REPLACE QUICKSILVER'S ITOH() FUNCTION
*
*  PARAM1: NUMERIC EXPRESSION TO CONVERT
*  PARAM2: IF <> 0, PAD RETURNED STRING WITH LEADING ZEROS TO THIS LENGTH
* RETURNS: A CHARACTER STRING WHICH IS THE HEXADECIMAL REPRESENTATION OF PARAM1
*
********************************************************************************
PRIVATE temp, result, power16

result = ""
power16 = 1
DO WHILE power16 < param1
   power16 = power16 * 16
ENDDO
IF power16 > param1
   power16 = power16 / 16
ENDIF
DO WHILE power16 >= 1
   IF power16 > param1
      result = result + "0"
   ELSE
      temp = INT(param1 / power16)
      IF temp < 10
         result = result + STR(temp,1)
      ELSE
         result = result + CHR(temp - 10 + ASC('A'))
      ENDIF
      param1 = param1 - power16 * temp
   ENDIF
   power16 = power16 / 16
ENDDO      

IF param2 > 0 .AND. LEN(result) < param2
   result = REPLICATE('0', param2 - LEN(result)) + result
ENDIF

RETURN result
*-- EOF: ItoH(param1, param2)


FUNCTION OverFlow   && Determine if a Value is a Numeric Overflow
PARAMETER num_value
*-----------------------------------------------------------------------------
* Synopsis:
* 
* OVERFLOW( <expN> )
* 
* The OVERFLOW() function determines if the <expN> expression is a numeric 
* overflow.  If the value is a numeric overflow or not a numeric value
* OVERFLOW() will return .T.  If the <expN> is a real numeric value OVERFLOW()
* will return .F.
* 
*-----------------------------------------------------------------------------
PRIVATE lc_result

IF .NOT. TYPE("num_value") $ "NF"
  lc_result = .T.
ELSE
  IF num_value < .9E308
    lc_result = .F.
  ELSE
    lc_result = .T.
  ENDIF
ENDIF

RETURN (lc_result)
*-- EOP: OverFlow( num_value )


FUNCTION Proper     && Convert a String to Proper Name Format
PARAMETERS param1
********************************************************************************
* PROGRAM: PROPER.PRG
*    DATE: 8/31/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE QUICKSILVER'S PROPER() FUNCTION
*
*  PARAM1: CHARACTER STRING TO CONVERT
* RETURNS: PARAM1 WITH THE FIRST LETTER OF EACH WORD CAPITALIZED
*
********************************************************************************
  PRIVATE position

  param1 = LOWER(param1)
  position = 2

  IF SUBSTR(param1,1,1) <> ' '
    param1 = STUFF(param1,1,1,UPPER(SUBSTR(param1,1,1)))
  ENDIF

  DO WHILE position <= LEN(param1)
    IF SUBSTR(param1,position-1,1) = ' '
      param1 = STUFF(param1,position,1,UPPER(SUBSTR(param1,position,1)))
    ENDIF
    position = position + 1
  ENDDO

RETURN param1
*-- EOF: Proper( param1 )


FUNCTION RAt        && AT() Starting from the Rigth Side
PARAMETERS msubstr, mstring
********************************************************************************
* PROGRAM: RAT.PRG
*    DATE: 8/11/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S RAT() FUNCTION
*
*  PARAM1: CHARACTER STRING EXPRESSION TO LOCATE
*  PARAM2: CHARACTER STRING EXPRESSION TO SEARCH
* RETURNS: STARTING POSITION OF LAST INSTANCE OF PARAM1 WITHIN PARAM2
*
********************************************************************************
PRIVATE mpos, mlen, lc_result, lc_talk

mlen = LEN(msubstr)
lc_result = LEN(mstring) - mlen

IF lc_result < 1
  lc_result = 0
ELSE
  DO WHILE lc_result > 0
    IF msubstr = SUBSTR(mstring, lc_result, mlen)
      EXIT
    ENDIF
    lc_result = lc_result - 1
  ENDDO
ENDIF

RETURN (lc_result)
*--EOF: RAt( msubstr, mstring )


FUNCTION Sec2Time   && Convert Seconds Past Midnight to Time String
PARAMETERS seconds
*-----------------------------------------------------------------------------
* Synopsis:
* 
* SEC2TIME( <expN> )
*                                
* The SEC2TIME() function converts an interger value <expN> representing
* the number of seconds from midnight to a time formatted string.  Values
* greater than 86400 seconds (the number of seconds on one day) will only
* show up as one day of time.  In invalid agrument type will return a NULL
* string.
* 
* See Also:  TIMEDIFF(), TIME2SEC()
* 
*-----------------------------------------------------------------------------
PRIVATE lc_lc_result, hrs, min, secs

*-- Check for valid type
  IF .NOT. TYPE( "seconds" ) $ "FIN"
    lc_result = ""
  ELSE

*-- Convert the seconds amount to seconds within one day if required
    IF seconds > 86400
      seconds = MOD( 86400, seconds )
    ENDIF

*-- Form the time formatted string
    hrs = LTRIM( STR( INT( seconds / 3600), 2) )
    IF LEN(hrs) = 1
      lc_result = "0" + hrs
    ELSE
      lc_result = hrs
    ENDIF

    min = LTRIM( STR( INT( (seconds - VAL(hrs)*3600)/60 ), 2) )
    IF LEN(min) = 1
      lc_result = lc_result + ":" + "0" + min
    ELSE
      lc_result = lc_result + ":" + min
    ENDIF

    secs = LTRIM( STR( (seconds - VAL(hrs)*3600 - VAL(min)*60), 2) )
    IF LEN(secs) = 1
      lc_result = lc_result + ":" + "0" + secs
    ELSE
      lc_result = lc_result + ":" + secs
    ENDIF

  ENDIF && End check for valid type


RETURN( lc_result )
*-- EOF: Sec2Time( seconds )


FUNCTION Selected   && Return WorkArea of Selected DBF
*-----------------------------------------------------------
* PROGRAM: SELECTED.PRG
*    DATE: 8/20/88
*      BY: Don Keller, StepIVward 1.0 
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S
*          SELECT() FUNCTION
*
* RETURNS: CURRENT SELECT AREA NUMBER
*
*-----------------------------------------------------------
  PRIVATE mtemp,mcnt

  mtemp = DBF()
  IF LEN(TRIM(mtemp)) = 0
    RETURN select()
  ENDIF

  mcnt = 1
  DO WHILE mcnt <= 10                    && Process all ten workareas
  IF DBF(mcnt) = mtemp .AND. LEN(DBF(mcnt)) = LEN(mtemp)
    RETURN mcnt
  ENDIF
  mcnt = mcnt + 1
ENDDO
*-- EOF: Selected()


FUNCTION SetEsc     && Replace Clipper SetEsc() Function
PARAMETERS param
*-----------------------------------------------------------
* PROGRAM: SETESC.PRG
*    DATE: 8/20/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S
*          SETCANCEL() FUNCTION
*
*   PARAM: NEW VALUE FOR SET ESCAPE (.T. => ON; .F. => OFF)
* RETURNS: CURRENT SET ESCAPE SETTING (.T. => ON; .F. =>
*          OFF)
*
*-----------------------------------------------------------

  IF TYPE('param') = 'L'
    IF param
      SET ESCAPE ON
    ELSE
      SET ESCAPE OFF
    ENDIF
  ENDIF

  IF set('ESCAPE') = 'ON'
  RETURN .T.
ELSE
  RETURN .F.
ENDIF
*-- EOF: SetEsc( param )


FUNCTION SetPrc     && Replace Clipper SETPRC() Function
PARAMETERS row, col
*-----------------------------------------------------------
* PROGRAM: SETPRC.PRG
*    DATE: 8/20/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S
*          SETPRC() FUNCTION
*          SETS INTERNAL ROW AND COLUMN FOR THE PRINTER
*
*  PARAM1: NEW INTERNAL ROW POSITION
*  PARAM1: NEW INTERNAL COLUMN POSITION
* RETURNS: .F.
*
*-----------------------------------------------------------
  _plineno = row
  _pcolno = col
RETURN .F.
*-- EOF: SetPrc( row, col )


FUNCTION ShowNdx    && Display Progess of INDEX
*-----------------------------------------------------------
* Display a BAR graph of the index in progess
*-----------------------------------------------------------
  trecs = RECCOUNT()
  @ 23,00 TO 23,79
  @ 23,00 TO 23,((RECNO()*80)/trecs)-1 DOUBLE
RETURN .T.
*-- EOF: ShowNdx()


FUNCTION SInkey     && Replace Quicksilver SINKEY() Function
*-----------------------------------------------------------
* PROGRAM: SINKEY.PRG
*    DATE: 8/31/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE QUICKSILVER'S 
*          SINKEY() FUNCTION
*
* RETURNS: ONE-BYTE STRING OF CHAR TYPED OR TWO-BYTE STRING 
*          FOR SPECIAL KEYS
*
*-----------------------------------------------------------
  PRIVATE value, qwerty, position

  value = INKEY()

  DO CASE

    CASE value >= 0 .AND. value <= 255 .AND. value <> 28
    RETURN CHR(value)

  CASE value = 28
    * F1
    RETURN CHR(255) + CHR(59)

  CASE value <= -1 .AND. value >= -9
    * F2 thru F10
    RETURN CHR(255) + CHR(59-value)

  CASE value <= -10 .AND. value >= -19
    * CTRL F1 thru CTRL F10
    RETURN CHR(255) + CHR(84-value)

  CASE value <= -20 .AND. value >= -28
    * SHIFT F1 thru SHIFT F9
    RETURN CHR(255) + CHR(64-value)


  CASE value >= -451 .AND. value <= -443
    * ALT 1 thru ALT 9
    RETURN CHR(255) + CHR(120+451+value)

  CASE value = -452
    * ALT 0
    RETURN CHR(255) + CHR(129)

  CASE value >= -435 .AND. value <= -410
    * ALT A thru ALT Z
    qwerty = "QWERTYUIOP    ASDFGHJKL     ZXCVBNM"
    position = AT(CHR(435+ASC('A')+value), qwerty)
    RETURN CHR(255) + CHR(position+15)

ENDCASE

RETURN ""
*-- EOF: SInkey()


FUNCTION StrSubst   && Replace a char with another throughout a string
PARAMETERS p_target, p_lookfor, p_repwith
*-----------------------------------------------------------
* Replaces a character with another throughout a string
*
* Synopsis:
*
* STRSUBST( <expC1>, <expC2>, <expC3> )
*
* Description:
*
* STRSUBST() replaces all occurances of the first character
* in <expC2> with the first character in <expC3> for string 
* <expC1>.  STRSUBST() will return a null string if all of
* the parameters are not string values.
*
* Examples:
*
*   ? STRSUBST( "This is a string", "i", "a" )
*     Thas as a strang
*
* See also: StrTran
*
* Author: Bill Ramos
*
* Warning: STRSUBST.BIN must be loaded before the call.  Use
*          the following procedure to load and release the
*          BIN file.
*
*          DO Load_Bin WITH "strsubst"
*          ...  Code to execute StrSubst()
*          DO Rele_Bin WITH "strsubst"
*
*-----------------------------------------------------------
PRIVATE lc_return

  lc_return = ""

  IF TYPE( "p_target" )  = "C" .AND. ;
     TYPE( "p_lookfor" ) = "C" .AND. ;
     TYPE( "p_repwith" ) = "C"

    IF glstrsubst && BIN file correctly LOADed with LOAD_BIN
      lc_return = CALL( "STRSUBST", ( p_target ), ;
                        p_lookfor, p_repwith )
    ENDIF
  ENDIF

RETURN( lc_return )
*-- EOF: StrSubst( p_target, p_lookfor, p_repwith )


FUNCTION StrTran    && Replace Clipper STRTRAN() Function
PARAMETERS ssearch, slocate, sreplace, repl_start, repl_max
*-----------------------------------------------------------
* PROGRAM: STRTRAN.PRG
*    DATE: 8/22/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S 
*          STRTRAN() FUNCTION
*
*  PARAM1: CHARACTER STRING TO SEARCH
*  PARAM2: CHARACTER STRING TO LOCATE
*  PARAM3: REPLACEMENT STRING
*  PARAM4: NUMBER OF FIRST OCCURRENCE TO REPLACE
*  PARAM5: NUMBER OF OCCURRENCES TO REPLACE
*
* RETURNS: THE UPDATED CHARACTER STRING
*
*-----------------------------------------------------------
  PRIVATE done, temp, incr, fnd_count, repl_count

  fnd_count = 0
  repl_count = 0
  temp = 1
  done = .F.
  DO WHILE .NOT. done
    temp = temp - 1 + AT(slocate, SUBSTR(ssearch,temp))
    IF temp = 0
      done = .T.
    ELSE
      fnd_count = fnd_count + 1
      IF fnd_count >= repl_start
        IF repl_max = -1 .OR. repl_count < repl_max
          repl_count = repl_count + 1
          ssearch = STUFF(ssearch, temp, LEN(slocate), sreplace)
          incr = LEN(sreplace)
        ELSE
          done = .T.
        ENDIF
      ELSE
        incr = LEN(slocate)
      ENDIF
      temp = temp + incr
      IF temp > LEN(ssearch) - LEN(slocate) + 1
        done = .T.
      ENDIF
    ENDIF
  ENDDO

RETURN ssearch
*-- EOF: StrTran(  ssearch, slocate, sreplace, repl_start, repl_max )


FUNCTION TalkMode   && Set and Query TALK Setting
PARAMETER howtoset
*-----------------------------------------------------------------------------
* 
* Synopsis:	
* 
* TALKMODE( <expL>/<anyother exp> )
* 
* This function sets the talk on/off mode based on <expL>.  The function 
* returns a logical value based on the setting of the set talk on/off 
* mode prior to execution of the function.  Using a non logical value
* for <expL> will return the current state of set talk on/off.
* 
* Examples:
* 
* Here is an example where TALKMODE() is used to set the talk mode on 
* and then restore the original talk mode setting:
* 
* oldmode = TALKMODE(.T.)      && Set talk mode on
* 
*         = TALKMODE(oldmode)  && Return mode to original setting
* 
* Here is an example which toggles the insert mode to the opposite value:
* 
*         = TALKMODE( .NOT. TALKMODE(0))
* 
*-----------------------------------------------------------------------------
PRIVATE lc_old

  IF SET("TALK") = "ON"
    SET TALK OFF
    lc_old = "ON"
  ELSE
    lc_old = "OFF"
  ENDIF

  IF TYPE( "howtoset" ) = "L"
    IF howtoset
      SET TALK ON
    ELSE
      SET TALK OFF
    ENDIF
  ENDIF

RETURN( lc_old )
*-- END: TalkMode( howtoset )


FUNCTION Time2Sec   && Convert a Time String to Seconds
PARAMETERS time_str
*-----------------------------------------------------------------------------
* Synopsis:
* 
* TIME2SEC( <expC> )
* 
* The TIME2SEC() function takes a properly formatted time string <expC> and
* returns an integer that represents the number of seconds after midnight.
* If the passed argument is a not the correct type, or not formatted 
* correctly, TIME2SEC will return a -1.
* 
* See Also:  TIMEDIFF(), SEC2TIME(), ISTIME()
* 
*-----------------------------------------------------------------------------
PRIVATE result

*-- Validate the time string values          
  IF IsTime( time_str )
    lc_result = (VAL( SUBSTR( time_str, 1, 2 ) ) * 3600) + ;
                (VAL( SUBSTR( time_str, 4, 2 ) ) * 60  ) + ;
                (VAL( SUBSTR( time_str, 7, 2 ) )       )
  ELSE
    lc_result = 1/0                  && Force numeric overflow
  ENDIF

RETURN( lc_result )
*-- EOF: Time2Sec( time_str )


FUNCTION TimeDiff   && Compute Difference of Two Time Strings
PARAMETERS start_time, end_time
*-----------------------------------------------------------------------------
* Synopsis:
* 
* TIMEDIFF( <expC1>, <expC2> )
* 
* The TIMEDIFF() function substracts time string <expC1> from time string
* <expC2> and returns a time formated string.  If an invalid type argument 
* is passed, then a NULL string is returned.  The time strings are based on
* the 24 hour clock.  If the time strings are not properly formed, then
* a NULL string is returned.
* 
* See Also: TIME2SEC(), SEC2TIME()
* 
*-----------------------------------------------------------------------------
PRIVATE s_hr, s_mn, s_sc, e_hr, e_mn, e_sc, r_hr, r_mn, r_sc
PRIVATE temp, lc_result                       

*-- Check for proper type of arguments
  IF TYPE( "start_time" ) <> "C" .OR. TYPE( "end_time" ) <> "C"
    lc_result = ""
  ELSE

*-- Convert time strings to integer components
    s_hr = VAL( SUBSTR(start_time,1,2) )
    s_mn = VAL( SUBSTR(start_time,4,2) )
    s_sc = VAL( SUBSTR(start_time,7,2) )

    e_hr = VAL( SUBSTR(end_time,1,2) )
    e_mn = VAL( SUBSTR(end_time,4,2) )
    e_sc = VAL( SUBSTR(end_time,7,2) )

*-- Validate the time string values
    IF .NOT. IsTime( start_time ) .OR. .NOT. IsTime( end_time )
      lc_result = ""
    ELSE
  
  
*-- Check to see if the timing spanned midnight
      r_hr = e_hr - s_hr
      IF r_hr < 0
        r_hr = r_hr + 24
      ENDIF

      r_mn = e_mn - s_mn
      IF r_mn < 0
        r_mn = r_mn + 60
        r_hr = r_hr - 1
      ENDIF

      r_sc = e_sc - s_sc
      IF r_sc < 0
        r_sc = r_sc + 60
        r_mn = r_mn -1
      ENDIF
      IF r_mn < 0
        r_mn = r_mn + 60
        r_hr = r_hr - 1
      ENDIF

      temp = LTRIM( STR(r_hr,2) )
      IF LEN( temp ) = 1
        lc_result = "0" + temp
      ELSE
        lc_result = temp
      ENDIF

      temp = LTRIM( STR(r_mn,2) )
      IF LEN( temp ) = 1
        lc_result = lc_result + ":" + "0" + temp
      ELSE
        lc_result = lc_result + ":" + temp
      ENDIF

      temp = LTRIM( STR(r_sc,2) )
      IF LEN( temp ) = 1
        lc_result = lc_result + ":" + "0" + temp
      ELSE
        lc_result = lc_result + ":" + temp
      ENDIF                                   

    ENDIF && End check fo valid time string
  ENDIF && End check for proper character type

RETURN( lc_result )
*-- EOF: TimeDiff( start_time, end_time )


FUNCTION TmpName    && Return an 8 character file root name
*----------------------------------------------------------
* Simulate FoxBASE's SYS(2) call for a temporary file
* name
* 
* Synopsis:
* 
* TMPNAME( )
* 
* Description:
* 
* Returns an 8 character string suitable as a temporary 
* file name.  The is based on the current system time.
* 
* Example:
* 

*   lc_tmpfile = TMPNAME() + ".tmp"
*   USE master
*   COPY TO (lc_tmpfile)
* 
* Warning: TMPNAME.BIN must be loaded before the call.  Use
*          the following procedure to load and release the
*          BIN file.
*
*          DO Load_Bin WITH "tmpname"
*          ...  Code to execute TmpName()
*          DO Rele_Bin WITH "tmpname"
*
*-----------------------------------------------------------
PRIVATE lc_return

  lc_return = ""

  IF gltmpname    && BIN file correctly LOADed with LOAD_BIN
    lc_return = CALL( "TMPNAME", SPACE( 8 ) )
  ENDIF

RETURN( lc_return )
*-- EOF: TmpName( )


FUNCTION Tone       && Replace Clipper TONE() Function
PARAMETERS mfreq, mduration
********************************************************************************
* PROGRAM: TONE.PRG
*    DATE: 8/11/88
*      BY: Don Keller, StepIVward 1.0
* PURPOSE: USER DEFINED FUNCTION TO REPLACE CLIPPER'S TONE() FUNCTION
*
*  PARAM1: FREQUENCY IN CYCLES PER SECOND FOR TONE
*  PARAM2: DURATION IN INCREMENTS OF 1/18 OF A SECOND
* RETURNS: .F.
*
********************************************************************************
  PRIVATE mbell, mconsole, mprinter

  mbell = set('BELL')
  mconsole = set('CONSOLE')
  mprinter = set('PRINTER')

  SET printer OFF
  SET CONSOLE ON
  SET BELL ON

  SET BELL TO mfreq, mduration
  ?? CHR(7)
  SET BELL TO

  IF mconsole = 'OFF'
    SET CONSOLE OFF
  ENDIF

  IF mbell <> 'ON'
    SET BELL OFF
  ENDIF

  IF mprinter = 'ON'
    SET printer ON
  ENDIF

RETURN .F.
*-- EOF: Tone( mfreq, mduration )


FUNCTION Updated    && Determine if User Updated GET Values After READ
*-------------------------------------------------------------
*-- Has data been updated
*-------------------------------------------------------------
  PRIVATE ll_return

  IF READKEY() > 256
    ll_return = .T.
  ELSE
    ll_return = .F.
  ENDIF

RETURN( ll_return )
*-- EOF: Updated()


FUNCTION WhatPara   && Comma Delimited String to Array
PARAMETER num_array, look4str
*-----------------------------------------------------------------------------
* Synopsis:
*
* WHATPARA <expA>, <expC>
*
* The WHATPARA function will create an array <expA> with the character values
* in the comma delimited string <expC>.  The WHATPARA functions main
* intent is a workaround for the seven parameter limit for UDF's.
* WHATPARA will return the number of array elements assigned.  WHATPARA
* will over write the existing array definition if it already exists.
*
* Examples:
*
*   howmany = WHATPARA(paralist,"1,2,foo bar,4")
*
*   The array PARALIST[1] = "1"
*             PARALIST[2] = "2"
*             PARALIST[3] = "foo bar"
*             PARALIST[4] = "4"
*   howmany = 4
*
* Restrictions:
*
*   Can only be used with dBASE > 1.1
*
*-----------------------------------------------------------------------------
   PRIVATE lc_delimit, lc_posit, lc_lenlook, lc_cnt, lc_args,;
           lc_str, lc_nxt_tok

   *-- Check for type of look4for string, terminate if not character
   IF TYPE("look4str") <> "C" .OR. TYPE("num_array") <> "C"
   RETURN(.F.)
ENDIF

*-- Check for the existance of the array
IF TYPE("&num_array[1]") <> "U"
   RELEASE &num_array
ENDIF

*-- Determine number of arguments, and assign it to LC_ARGS
lc_delimit = ","
lc_posit = 1
lc_lenlook = LEN(look4str)
lc_cnt = 0
DO WHILE lc_posit < lc_lenlook
   IF SUBSTR(look4str, lc_posit, 1) = lc_delimit
      lc_cnt = lc_cnt + 1
   ENDIF
   lc_posit = lc_posit + 1
ENDDO
lc_args = lc_cnt + 1

*-- Declare the return array NUM_ARRAY
PUBLIC ARRAY &num_array[lc_args]

*-- Put the elements in place
lc_posit = 1
lc_str = look4str
lc_nxt_tok = AT(lc_delimit,lc_str)
DO WHILE lc_nxt_tok > 0
   STORE LEFT(lc_str, lc_nxt_tok-1) TO &num_array[lc_posit]
   lc_str = SUBSTR(lc_str, lc_nxt_tok + 1 )
   lc_nxt_tok = AT(lc_delimit,lc_str)
   lc_posit = lc_posit + 1
ENDDO
STORE LEFT(lc_str, LEN(lc_str)) TO &num_array[lc_posit]

RETURN (lc_args)
*-- EOF: WhatPara( num_array, look4str )


PROCEDURE Key_Vars  && Set INKEY() and READKEY() Values to Variables
*----------------------------------------------------------------------------
* Enumerate the key values for LASTKEY() and INKEY() functions
*
* To check for the Escape key after the INKEY()
* 
*   ln_key = INKEY(0)   && Wait for any key press
*   IF ln_key = kn_Esc  && Escape was pressed
*     DO esc_hand
*   ENDIF
*
*----------------------------------------------------------------------------
IF TYPE("kn_Esc") = "U"

  PUBLIC kn_End     , kn_Tab     , kn_Enter   , kn_CtrlEnd , kn_CtrlY   , ;
       kn_Home    , kn_Esc     , kn_CtrlHme , kn_CtrlPDn , kn_CtrlPUp , ;
       kn_Space   , kn_BakSpce , kn_RghtArw , kn_UpArw   , kn_LeftArw , ;
       kn_DownArw , kn_PgDn    , kn_PgUp    , kn_F1      , kn_Del     , ;
       kn_CtrLArw , kn_CtrRArw , kn_f7      , kn_ShftF7

  kn_End     =   2      && Ctrl-B
  kn_Tab     =   9      && Ctrl-I
  kn_Enter   =  13      && Ctrl-M
  kn_CtrlEnd =  23      && Ctrl-W
  kn_CtrlY   =  25
  kn_Home    =  26      && Ctrl-Z
  kn_Esc     =  27      && Ctrl-[
  kn_CtrlHme =  29      && Ctrl-]
  kn_CtrlPDn =  30      && Ctrl-PgDn
  kn_CtrlPUp =  31      && Ctrl-PgUp
  kn_Space   =  32
  kn_BakSpce = 127
  kn_RghtArw =   4      && Ctrl-D
  kn_UpArw   =   5      && Ctrl-E
  kn_LeftArw =  19      && Ctrl-S
  kn_DownArw =  24      && Ctrl-X
  kn_PgDn    =   3      && Ctrl-C
  kn_PgUp    =  18      && Ctrl-R
  kn_F1      =  28      && Ctrl-\
  kn_Del     =   7      && Ctrl-G
  kn_CtrLArw =   1      && Ctrl-A
  kn_CtrRArw =   6      && Ctrl-F
  kn_F7      =  -6
  kn_ShftF7   = -26

*----------------------------------------------------------------------------
* Enumerate the key values for READKEY()
*
* To check to see if data has changed
*
*   IF READKEY() >= rn_updated   && Data has changed
*     REPLACE name WITH m->name
*   ENDIF
*
* To check for page down regardless of data change
*
*   ln_readkey = READKEY()
*   IF ln_readkey = rn_PgDn .OR. ln_readkey = rn_PgDn+rn_Updated
*     DO pgdn_hand
*   ENDIF
*
*----------------------------------------------------------------------------
  PUBLIC rn_Updated , rn_LeftArw , rn_BakSpce , rn_RghtArw , rn_CtrLArw , ;
       rn_CtrRArw , rn_UpArw   , rn_DownArw , rn_PgUp    , rn_PgDn    , ;
       rn_Esc     , rn_CtrlEnd , rn_Enter   , rn_EnterA  , rn_CtrlHme , ;
       rn_CtrlPUp , rn_CtrlPDn , rn_F1

  rn_Updated = 256      && Add to rn_key value for updated condition
  rn_LeftArw =   0      && Includes Ctrl-S and Ctrl-H - backward one character
  rn_BakSpce =   0      && backward one character                                   
  rn_RghtArw =   1      && Includes Ctrl-D and Ctrl-L - forward one character
  rn_CtrLArw =   2      && Ctrl-Left Arrow, includes Ctrl-A - previous word
  rn_CtrRArw =   3      && Ctrl-Right Arrow, includes Ctrl-F - next word
  rn_UpArw   =   4      && Includes Ctrl-E and Ctrl-K - backward one field
  rn_DownArw =   5      && Includes Ctrl-J and Ctrl-X - forward one field
  rn_PgUp    =   6      && Includes Ctrl-R - backward one screen
  rn_PgDn    =   7      && Includes Ctrl-C - forward one screen
  rn_Esc     =  12      && Includes Ctrl-Q - Terminate w/o save
  rn_CtrlEnd =  14 + rn_updated  && Includes Ctrl-W - Terminate w/save
  rn_Enter   =  15      && Includes Ctrl-M  RETURN of fill last record
  rn_EnterA  =  16      && Enter at the beginning of a record in APPEND
  rn_CtrlHme =  33      && Ctrl-Home - Menu display toggle
  rn_CtrlPUp =  34      && Ctrl-PgUp - Zoom Out
  rn_CtrlPDn =  35      && Ctrl-PgDn - Zoom In
  rn_F1      =  36      && Help function key
ENDIF && TYPE("kn_Esc") = "U"

RETURN
*-- EOP: Key_Vars.prg


PROCEDURE Load_Bin  && Load a BIN File and Set the Load Status
PARAMETER binfile
*-----------------------------------------------------------------------------
* Synopsis:
* 
* DO LOAD_BIN WITH <expF>
* 
* LOAD_BIN loads a BIN file is the file exists and creates a global variable
* that is made up of the prefix "GL" and a suffix the same as the BIN file's
* root name.  The global variable is set to true (.T.) is properly loaded, or
* it's set to false (.F.) is the BIN file was not loaded.  A full file path 
* can be specified for the BIN file.
* 
* If the BIN file is already loaded, noted by the global variable set to
* true, then the global variable will return true with no further action.
* 
* Example:
* 
*   DO Load_Bin WITH "scroll"
* 
* This will load the BIN file SCROLL into memory.  The global variable
* named GLSCROLL will be created and set to true (.T.)
* 
* See Also:  RELE_BIN
* 
*-----------------------------------------------------------------------------
PRIVATE lc_found, lc_dot, lc_back, lc_name

*-- Make sure bin file exists
IF "." $ binfile
  IF FILE( binfile )
    lc_found = .T.
  ELSE
    lc_found = .F.
  ENDIF
ELSE
  IF FILE( binfile + ".BIN" )
    lc_found = .T.
  ELSE
    lc_found = .F.
  ENDIF
ENDIF

*-- Make PUBLIC variable to denote BIN file is loaded
lc_back = RAT("\", binfile)
IF "." $ binfile                                   
  lc_dot  =  AT(".", binfile)
  lc_name = SUBSTR(binfile, lc_back+1, lc_dot-lc_back+1)
ELSE
  lc_name = SUBSTR(binfile, lc_back+1, LEN(binfile)-lc_back)
ENDIF

IF TYPE( "GL" + lc_name ) = "U"
  PUBLIC gl&lc_name
  gl&lc_name = .F.
ENDIF

IF .NOT. gl&lc_name .AND. lc_found
  LOAD &binfile
  gl&lc_name = .T.
ENDIF

RETURN
*-- EOP: Load_Bin WITH binfile


PROCEDURE LoadBins  && Load StepIVWard BIN modules
*-----------------------------------------------------------
* Load StepIVWard BIN modules
*-----------------------------------------------------------
DO load_bin WITH "CREATE"
DO load_bin WITH "CURDIR"
DO load_bin WITH "GETDRIVE"
DO load_bin WITH "STRSUBST"
DO load_bin WITH "TMPNAME"
RETURN   
*-- EOP: LoadBins


PROCEDURE Rele_Bin  && Release a BIN File and Clear Load Status
PARAMETER binfile
*-----------------------------------------------------------------------------
* Synopsis:
* 
* DO RELE_BIN WITH <expF>
* 
* RELE_BIN will release a BIN file from memory, based on the global variable
* set by LOAD_BIN.  When released, the global variable will be set to false.
* A full file path may be used, but only the root file name is required to
* release a BIN file from memory.
* 
* Example:
* 
*   DO RELE_BIN WITH "scroll"
* 
* This will release the BIN file named scroll, and set the global variable,
* GLSCROLL to false (.F.).
* 
* See Also:  LOAD_BIN
* 
*-----------------------------------------------------------------------------
PRIVATE lc_dot, lc_back, lc_name

*-- Make PUBLIC variable to denote BIN file is loaded
lc_back = RAT("\", binfile)
IF "." $ binfile                                   
  lc_dot  =  AT(".", binfile)
  lc_name = SUBSTR(binfile, lc_back+1, lc_dot-lc_back+1)
ELSE
  lc_name = SUBSTR(binfile, lc_back+1, LEN(binfile)-lc_back)
ENDIF

IF TYPE( "GL" + lc_name ) = "U"
  PUBLIC gl&lc_name
  gl&lc_name = .F.
ENDIF

IF gl&lc_name
  RELEASE MODULE &lc_name
  gl&lc_name = .F.
ENDIF

RETURN
*-- EOP: Rele_Bin WITH binfile


PROCEDURE Rele_Vars && Release the INKEY()/READKEY() var definitions
*-------------------------------------------------------------------
* Releases the variables defined to make code readable in KEY_VARS
*-------------------------------------------------------------------
RELEAS kn_End     , kn_Tab     , kn_Enter   , kn_CtrlEnd , kn_CtrlY   , ;
       kn_Home    , kn_Esc     , kn_CtrlHme , kn_CtrlPDn , kn_CtrlPUp , ;
       kn_Space   , kn_BakSpce , kn_RghtArw , kn_UpArw   , kn_LeftArw , ;
       kn_DownArw , kn_PgDn    , kn_PgUp    , kn_F1      , kn_Del     , ;
       kn_CtrLArw , kn_CtrRArw , kn_f7      , kn_ShftF7

RELEAS rn_Updated , rn_LeftArw , rn_BakSpce , rn_RghtArw , rn_CtrLArw , ;
       rn_CtrRArw , rn_UpArw   , rn_DownArw , rn_PgUp    , rn_PgDn    , ;
       rn_Esc     , rn_CtrlEnd , rn_Enter   , rn_EnterA  , rn_CtrlHme , ;
       rn_CtrlPUp , rn_CtrlPDn , rn_F1
RETURN
*-- EOP: Rele_Vars


PROCEDURE ReleBins  && Release StepIVWard BIN modules
*-----------------------------------------------------------
* Release StepIVWard BIN modules
*-----------------------------------------------------------
DO rele_bin WITH "CREATE"
DO rele_bin WITH "CURDIR"
DO rele_bin WITH "GETDRIVE"
DO rele_bin WITH "STRSUBST"
DO rele_bin WITH "TMPNAME"
RETURN
*-- EOP: ReleBins


PROCEDURE Shadow    && Display Shadow Effect
PARAMETER x1,y1,x2,y2,dfill
*-------------------------------------------------------------

  IF dfill                              && use fill command not window

    @ x1+1,y1+2 FILL TO x2+1,y2+2 COLOR n+/n

  ELSE                                  && define shadow window

    DEFINE WINDOW shadow_win FROM x1+1,y1+2 TO x2+1,y2+2 none color n+/n
    ACTIVATE WINDOW shadow_win

  ENDIF

RETURN
*-- EOP: Shadow


PROCEDURE ShadowG   && Display Shadow Effect That Grows
PARAMETER x1,y1,x2,y2
*-------------------------------------------------------------
  x0 = x2+1
  y0 = y2+2
  dx = 1
  dy = (y2-y1) / (x2-x1)
  DO WHILE x0 <> x1 .OR. y0 <> y1+2

    @ x0,y0 FILL TO x2+1,y2+2 COLOR n+/n
    x0 = IIF(x0<>x1,x0 - dx,x0)
    y0 = IIF(y0<>y1+2,y0 - dy,y0)
    y0 = IIF(y0<y1+2,y1+2,y0)
  ENDDO

RETURN
*-- EOP: ShadowG


PROCEDURE TalkOn    && Set TALK ON if Originally ON
  IF TYPE( "zz_talk" ) = "U"
    SET TALK ON
  ELSE
    IF zz_talk = "ON"
      SET TALK ON
    ENDIF
    RELEASE zz_talk
  ENDIF
RETURN
*-- EOP: TalkOn


PROCEDURE TalkOff   && Set TALK OFF and Save Previous State
  PUBLIC zz_talk
  IF set("TALK") = "ON"
    SET TALK OFF
    zz_talk = "ON"
  ELSE
    zz_talk = "OFF"
  ENDIF
RETURN
*-- EOP: TalkOFF

*-----------------------------------------------------------------------*
*                   UDS - USER DEFINED SURFACES FOLLOW                --*
*-----------------------------------------------------------------------*



