PROCEDURE Layout
PARAMETERS pc_surface, pc_file
*----------------------------------------------------------------------------
* DESCRIPTION
*   Layout will display a OACC Layout popup based on the records contained
*   in Layout.DBF.  Layout will create a popup bar for each record where
*   the SURFACE field matches the value of <pc_surface>.  Layout will
*   run the program name stored in the field, PRG2RUN, when the user selects
*   the popup bar.  Layout will pass both the surface, <pc_surface>, and
*   the design file name, <pc_file>, parameters to the PRG2RUN program.
*
*   Layout will look for the Layout.DBF in the current directory, the
*   dBASE search PATH, and the dBASE HOME() directory.  If Layout cannot
*   locate Layout.DBF, it will display an error message and return back 
*   to the design surface.
*
*   Layout will have TALK OFF when it calls the PRG2RUN and return the
*   TALK back to its original value when it returns to the surface.
*   Layout will clear the popup from the screen and close Layout.dbf
*   before calling the PRG2RUN.
*----------------------------------------------------------------------------
  PRIVATE ll_talk, ll_lfound, lc_prefix

  *---------------
  *-- Set talk off
  *---------------
  IF SET( "TALK" ) = "ON"
    SET TALK OFF
    ll_talk = .T.
  ELSE
    ll_talk = .F.
  ENDIF

  *-------------------------------------------
  *-- Try to find Layout.DBF before opening it
  *-------------------------------------------
  ll_lfound = .F.                       && Track if Layout.DBF was found
  IF FILE( "Layout.DBF" )
    lc_prefix = ""
    ll_lfound = .T.
  ELSE
    lc_prefix = HOME()
    IF FILE( lc_prefix + "Layout.DBF" )
      ll_found = .T.
    ENDIF
  ENDIF

  *---------------------------------------------
  *-- Error out, if the Layout file is not found
  *---------------------------------------------
  IF .NOT. ll_lfound                    
    DO Err_Box WITH "File not found: Layout.dbf", .t.
    IF ll_talk
      SET TALK ON
    ENDIF
    RETURN
  ENDIF

  *-------------------------------------------------------------------
  *-- Open the Layout file and set the key range to the surface values
  *-------------------------------------------------------------------
  ll_exact = SET( "EXACT" ) = "ON"      && Save EXACT state before
  SET EXACT OFF                         && turning EXACT off

  lc_alias = ALIAS()                    && Capture the current ALIAS
  SELECT SELECT()                       && Grab an available workarea
  USE ( lc_prefix + "Layout" ) ORDER dorder
  SET KEY TO pc_surface                 && Look at only this surface
  GO TOP

  *----------------------------------------------------
  *-- Make sure there are menu options for this surface
  *----------------------------------------------------
  IF EOF()
    DO Err_Box WITH "Layout.dbf has no menu options for this surface", .t.
    IF ll_exact
      SET EXACT ON
    ENDIF
    USE
    IF .NOT. ISBLANK( lc_alias )
      SELECT ( lc_alias )
    ENDIF
    IF ll_talk
      SET TALK ON
    ENDIF
    RETURN
  ENDIF

  *-------------------------------------------------------------------------
  *-- Calculate the popup dimensions based on the number of bars and prompts
  *-------------------------------------------------------------------------
  CALCULATE MAX( LEN( TRIM( prompt ) ) ), CNT() TO ln_maxbar, ln_count
  ln_top = 5                            && Row position for the popup
  ln_left = 39 - ( ln_maxbar + 6 ) / 2  && Column for popup to center it
  ln_bottom = ln_top + ln_count + 1     && Bottom row for the popup
  ln_right = ln_left + ln_maxbar + 6    && Right side including two spaces
                                        && on each side of the longest prompt

  DEFINE POPUP Layout FROM ln_top, ln_left TO MIN( ln_bottom, 21 ), ln_right

  *------------------------
  *-- Define the popup bars
  *------------------------
  ln_bar = 1
  SCAN
    DEFINE BAR ln_bar OF Layout PROMPT "  " + TRIM( Prompt )
    ln_bar = ln_bar + 1
  ENDSCAN
  ON SELECTION POPUP Layout DEACTIVATE POPUP

  *----------------------------------
  *-- Activate the popup for the user
  *----------------------------------
  SAVE SCREEN TO Layout
  DO ShadowG WITH ln_top, ln_left, MIN( ln_bottom, 21 ), ln_right
  ACTIVATE POPUP Layout
  RESTORE SCREEN FROM Layout
  RELEASE SCREEN Layout

  *-----------------------------------------------
  *-- Determine which PRG2RUN for the selected bar
  *-----------------------------------------------
  ln_bar = BAR()
  lc_prg2run = ""
  IF ln_bar > 0    
    GO TOP
    SKIP ln_bar - 1
    lc_prg2run = TRIM( prg2run )
  ENDIF

  *--------------------------------------------------------
  *-- Close down the Layout file before running the PRG2RUN
  *--------------------------------------------------------
  USE
  IF .NOT. ISBLANK( lc_alias )
    SELECT ( lc_alias )
  ENDIF
  RELEASE POPUP Layout

  IF .NOT. ISBLANK( lc_prg2run )
    DO &lc_prg2run WITH pc_surface, pc_file
  ENDIF

  *--------------------------------------
  *-- Restore the rest of the environment
  *--------------------------------------
  IF ll_exact
    SET EXACT ON
  ENDIF
  IF ll_talk
    SET TALK ON
  ENDIF

RETURN
*-- EOP: Layout WITH pc_surface, pc_file


